#!/usr/bin/env python

"""
Webware adapter.
"""

import WebStack.Webware

# For Webware releases later than 0.8.1, employ special URLParsers in contexts
# for each application in the application server; such parsers create servlets
# instead of having servlet factories do that work.

try:
    from WebKit.URLParser import URLParser

except ImportError:

    # NOTE: Using Webware 0.8.1 or earlier. Assume that this really is the case.

    pass

else:
    class WebStackURLParser(URLParser):

        """
        A custom URL parser which provides access to application-specific resources.
        Override the 'parse' method for more precise control of servlet
        instantiation.
        """

        def __init__(self, resource):

            """
            Initialise the parser object with the given root application-specific
            'resource'.
            """

            self.webstack_resource = resource

        def parse(self, trans, requestPath):

            """
            For the given Webware transaction, 'trans', override the usual servlet
            factory mechanism and return a servlet which will provide access to the
            application-specific resources.
            The 'trans' object - a Webware transaction - is not given to the servlet
            since such information is available when the 'respond' method is invoked
            on the servlet.
            The provided 'requestPath' object is not used, since this information
            should be available elsewhere.
            """

            return WebStackServlet(self.webstack_resource)

# For Webware 0.8.1 and earlier, employ servlet factories and servlets.

from WebKit.ServletFactory import ServletFactory
from WebKit.Servlet import Servlet

class WebStackServletFactory(ServletFactory):

    """
    A servlet factory object producing servlets which provide access to
    application-specific resources.
    """

    def __init__(self, application, resource, file_extensions):

        """
        Initialise the servlet factory with the Webware 'application' and the
        WebStack root application-specific 'resource'. The 'file_extensions'
        specified indicate for which files this factory is invoked.
        """

        ServletFactory.__init__(self, application)
        self.webstack_resource = resource
        self.file_extensions = file_extensions

    def uniqueness(self):

        """
        Return "file" uniqueness - probably the most appropriate response.
        """

        return "file"

    def extensions(self):

        """
        Return the file extensions supported by this factory.
        """

        return self.file_extensions

    def servletForTransaction(self, trans):

        """
        Return a servlet which will provide access to the application-specific
        resources. The 'trans' object - a Webware transaction - is not given to
        the servlet since such information is available when the 'respond'
        method is invoked on the servlet.
        """

        return WebStackServlet(self.webstack_resource)

# Servlets are common to both solutions.

class WebStackServlet(Servlet):

    "A servlet which dispatches transactions to application-specific resources."

    def __init__(self, resource):

        "Initialise the servlet with an application-specific 'resource'."

        Servlet.__init__(self)
        self.webstack_resource = resource

    def respond(self, trans):

        """
        Respond to the incoming transaction, 'trans', by dispatching to the
        application-specific resource.
        """

        new_trans = WebStack.Webware.Transaction(trans)
        self.webstack_resource.respond(new_trans)
        new_trans.commit()

# vim: tabstop=4 expandtab shiftwidth=4
